%{
/* C++ string header, for string ops below */
#include <string>

/* Implementation of yyFlexScanner */ 
#include "mc_scanner.hpp"
#undef  YY_DECL
#define YY_DECL int MC::MC_Scanner::yylex( MC::MC_Parser::semantic_type * const lval, MC::MC_Parser::location_type *loc )

/* typedef to make the returns for the tokens shorter */
using token = MC::MC_Parser::token;

/* define yyterminate as this instead of NULL */
#define yyterminate() return( token::END )

/* msvc2010 requires that we exclude this header file. */
#define YY_NO_UNISTD_H

/* update location on matching */
#define YY_USER_ACTION loc->step(); loc->columns(yyleng);

%}

%option debug
%option nodefault
%option yyclass="MC::MC_Scanner"
%option noyywrap
%option c++

%%
%{          /** Code executed at the beginning of yylex **/
            yylval = lval;
%}


#	{
	std::cout << "###";
	return token::COMMENT;
}

"\r\n"	{
	return token::NEWLINE;
	}
	
"\n"	{
	std::cout << "\n";
	return token::NEWLINE;
	}
	
"\t"	{
	return token::TAB;
	}

"=" {
	return token::ASSIGN;
	}

" " {
	return token::SPACE;
	}
	

^[^#][a-zA-Z0-9\._]+:	{
	std::string s(yytext);
	std::cout << s;
	yylval->build< std::string >( s.substr(0,s.size()-1) );
	return token::RULE_NAME;
}

\"[^\a\b\f\n\r\t\v"]*\"	{
	std::string s(yytext);
	std::cout << s.substr(1,s.size()-2);
	yylval->build< std::string >( s.substr(1,s.size()-2) );
	return token::QUOTED_STRING;
}

[^\a\b\f\n\r\t\v "]+	{
	yylval->build<std::string>(yytext);
	std::cout << " " << yytext << " ";
	return token::STRING;
}

%%


