%skeleton "lalr1.cc"
%require  "3.0"
%debug 
%defines 
%define api.namespace {MC}
/**
 * bison 3.3.2 change
 * %define parser_class_name to this, updated
 * should work for previous bison versions as 
 * well. -jcb 24 Jan 2020
 */
%define api.parser.class {MC_Parser}

%code requires{
   namespace MC {
      class MC_Driver;
      class MC_Scanner;
   }

// The following definitions is missing when %locations isn't used
# ifndef YY_NULLPTR
#  if defined __cplusplus && 201103L <= __cplusplus
#   define YY_NULLPTR nullptr
#  else
#   define YY_NULLPTR 0
#  endif
# endif

}

%parse-param { MC_Scanner  &scanner  }
%parse-param { MC_Driver  &driver  }

%code{
   #include <iostream>
   #include <cstdlib>
   #include <fstream>
   
   /* include for all driver functions */
   #include "mc_driver.hpp"

#undef yylex
#define yylex scanner.yylex
}

%define api.value.type variant
%define parse.assert


%token               END    0     "end of file"
%token <std::string> STRING
%token <std::string> QUOTED_STRING
%token <std::string> RULE_NAME
%token NEWLINE
%token TAB SPACE
%token COMMENT
%token ASSIGN

%locations
%start program

%%

program:
	block	{ std::cout << "x"; }
	| comment	{ std::cout << "z"; }
	| program block 	{ std::cout << "y"; }
	| program comment 	{ std::cout << "a"; }
	| END

block:
	rule	{ std::cout << "b"; }
	| assignment	{ std::cout << "c"; }
	
comment:
	COMMENT SPACE comment_eater NEWLINE	{ std::cout << "-"; }
	| COMMENT NEWLINE	{ std::cout << "--"; }

comment_eater:
	STRING	{ std::cout << $1; }
	| QUOTED_STRING { std::cout << $1; }
	| comment_eater SPACE STRING	{ std::cout << $3; }
	| comment_eater SPACE QUOTED_STRING { std::cout << $3; }

rule:
	RULE_NAME SPACE sources NEWLINE commands { std::cout << "target: " << $1; }

sources:
	sources STRING { std::cout << "source: " << $2; }
	| sources QUOTED_STRING { std::cout << "source: " << $2; }
	| STRING { std::cout << " source: " << $1; }
	| QUOTED_STRING { std::cout << " source: " << $1; }
	| %empty

commands:
	commands NEWLINE STRING { std::cout << "command: " << $3; }
	| commands NEWLINE QUOTED_STRING { std::cout << "command: " << $3; }
	| TAB STRING { std::cout << "command: " << $2; }
	| TAB QUOTED_STRING { std::cout << "command: " << $2; }
	| %empty

assignment:
	STRING SPACE ASSIGN SPACE STRING NEWLINE	{ std::cout << "assignment: " << $1 << " " << $5; }
	| STRING SPACE ASSIGN SPACE QUOTED_STRING	{ std::cout << "assignment: " << $1 << " " << $5; }
	| STRING SPACE ASSIGN	{ std::cout << "assignment: " << $1 << " cleared"; }

%%


void 
MC::MC_Parser::error( const location_type &l, const std::string &err_message )
{
   std::cerr << "Error: " << err_message << " at " << l << "\n";
}
